///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CStringBuilder_CPP
#define _CStringBuilder_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "CStringBuilder.H"
#include "../../@Libraries/NSWFL/NSWFL.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::String;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef _USE_GLOBAL_MEMPOOL
#include "../CMemPool/CMemPool.H"
extern CMemPool *pMem; //pMem must be defined and initalized elsewhere.
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CStringBuilder CStringBuilder::operator+= (CStringBuilder &pStr)
{
	this->Append(&pStr);
	return this;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CStringBuilder CStringBuilder::operator+ (CStringBuilder &pStr)
{
	CStringBuilder newString(this);
	newString.Append(&pStr);
	return CStringBuilder(&newString);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CStringBuilder::operator== (CStringBuilder &pStr)
{
	return(this->StrCmp(&pStr) == 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CStringBuilder::operator> (CStringBuilder &pStr)
{
	return(this->StrCmp(&pStr) > 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CStringBuilder::operator< (CStringBuilder &pStr)
{
	return(this->StrCmp(&pStr) < 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CStringBuilder::operator>= (CStringBuilder &pStr)
{
	return(this->StrCmp(&pStr) >= 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CStringBuilder::operator<= (CStringBuilder &pStr)
{
	return(this->StrCmp(&pStr) <= 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CStringBuilder::operator!= (CStringBuilder &pStr)
{
	return(this->StrCmp(&pStr) != 0);
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

char &CStringBuilder::operator[] (unsigned int i)
{
	return this->Buffer[i];
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

char &CStringBuilder::operator[] (int i)
{
	return this->Buffer[i];
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Operator: char*
*/
CStringBuilder::operator const char*(void)
{
	return this->Buffer;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Operator: char*
*/
CStringBuilder::operator char*(void)
{
	return this->Buffer;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Operator: int
*/
CStringBuilder::operator int(void)
{
	return this->Length;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Default constructor.
*/
CStringBuilder::CStringBuilder(void)
{
	this->Initialize(CSTRINGBUILDER_GRANDULARITY);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Constructor, setting the initial size of the buffer.
*/
CStringBuilder::CStringBuilder(int InitialSize)
{
	this->Initialize(InitialSize);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Constructor, setting the initial value (and therfor the size) of the buffer.
*/
CStringBuilder::CStringBuilder(const char *String)
{
	int iLength = (int)strlen(String) + 10;
	this->Initialize(iLength);
	this->Append(String, iLength - 10);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Constructor, setting the initial value of the buffer as well as the initial size.

	[InitialSize] should (but does not have to) be larger than [Size].
*/
CStringBuilder::CStringBuilder(const char *String, int Size, int InitialSize)
{
	this->Initialize(InitialSize);
	this->Append(String, Size);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Constructor, setting the initial value of the buffer.
*/
CStringBuilder::CStringBuilder(const char *String, int Size)
{
	this->Initialize(Size);
	this->Append(String, Size);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Constructor, clone another instance of the CStringBuilder class.
*/
CStringBuilder::CStringBuilder(CStringBuilder *pSource)
{
	this->Initialize(pSource->Alloc);
	this->Append(pSource);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Destructor.
*/
CStringBuilder::~CStringBuilder()
{
	this->Destroy();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Default Initializer.
	
	DO NOT call if a constructor was called unless you previously Destroy()d.
*/
bool CStringBuilder::Initialize(void)
{
	return this->Initialize(CSTRINGBUILDER_GRANDULARITY);
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Initializer, sets the allocate ahead size.
	
	DO NOT call if a constructor was called unless you previously Destroy()d.
*/
bool CStringBuilder::Initialize(const int iInAllocAhead)
{
	this->AllocAhead = iInAllocAhead;
	
	#ifdef _USE_GLOBAL_MEMPOOL
		if((this->Buffer = (char *) pMem->Allocate(sizeof(char), this->AllocAhead + 1)) == NULL)
		{
			return false;
		}
	#else
		if((this->Buffer = (char *) calloc(sizeof(char), this->AllocAhead + 1)) == NULL)
		{
			return false;
		}
	#endif

	this->Alloc = this->AllocAhead;
	this->Length = 0;

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Destructor/uninitializer.
	
	Used to destroy the string buffer before the class falls out of scope.
*/
void CStringBuilder::Destroy(void)
{
	this->AllocAhead = 0;
	this->Alloc = 0;
	this->Length = 0;

	if(this->Buffer)
	{
		#ifdef _USE_GLOBAL_MEMPOOL
			pMem->Free(this->Buffer);
		#else
			free(this->Buffer);
		#endif
	}

	this->Buffer = NULL;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns the zero-based index of the first non-whitespace character in the string buffer.
*/
int CStringBuilder::FirstCharPos(void)
{
	return GetFirstCharPos(this->Buffer, this->Length);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns the zero-based index of the last non-whitespace character in the string buffer.
*/
int CStringBuilder::LastCharPos(void)
{
	return GetLastCharPos(this->Buffer, this->Length);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends to the string buffer using standard formatting. Exactly like sprintf("", ...)
*/
int CStringBuilder::AppendF(const char *sFormat, ...)
{
	va_list ArgList;
	va_start(ArgList, sFormat);

	int iMemoryRequired = _vscprintf(sFormat, ArgList);

	#ifdef _USE_GLOBAL_MEMPOOL
		char *sBuf = (char *) pMem->Allocate(sizeof(char), iMemoryRequired + 1);
	#else
		char *sBuf = (char *) calloc(sizeof(char), iMemoryRequired + 1);
	#endif

	int iSz = _vsprintf_s_l(sBuf, iMemoryRequired + 1, sFormat, NULL, ArgList);
	va_end(ArgList);

	this->Append(sBuf, iSz);
	this->Terminate();

	#ifdef _USE_GLOBAL_MEMPOOL
		pMem->Free(sBuf);
	#else
		free(sBuf);
	#endif

	return this->Length;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Replaces a range of bytes with the specified bytes.
*/
int CStringBuilder::ReplaceRange(int iBegin, int iEnd, const char *sWith, int iWithSz)
{
	int iNewSz = this->Length + (iWithSz - (iEnd - iBegin));
	int iGapSize = (this->Length - iNewSz);
	int iPos = 0;

	if(iNewSz >= this->Alloc)
	{
		this->IncreaseSize(iNewSz + 1);
	}

	if(iGapSize < 0)
	{
		iGapSize = abs(iGapSize);

		for(iPos = iNewSz; iPos > iEnd && (iPos - iGapSize) >= 0; iPos--)
		{
			this->Buffer[iPos] = this->Buffer[iPos - iGapSize];
		}
		memcpy(this->Buffer + iBegin, sWith, iWithSz);
	}
	else if(iGapSize > 0)
	{
		memcpy(this->Buffer + iBegin, sWith, iWithSz);

		for(iPos = iBegin + iWithSz; iPos < iNewSz; iPos++)
		{
			this->Buffer[iPos] = this->Buffer[iPos + iGapSize];
		}

		this->Buffer[iPos] = '\0';
	}
	else{
		memcpy(this->Buffer + iBegin, sWith, iWithSz);
	}

	this->Length = iNewSz;

	return this->Length;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends data to the data buffer from a windows textbox.
*/
bool CStringBuilder::AppendFromTextBox(HWND hTextBox)
{
	int iTextLen = (GetWindowTextLength(hTextBox));

	#ifdef _USE_GLOBAL_MEMPOOL
		char *sBuf = (char *) pMem->Allocate(iTextLen + 1, sizeof(char));
	#else
		char *sBuf = (char *) calloc(iTextLen + 1, sizeof(char));
	#endif

	if(sBuf != NULL)
	{
	    iTextLen = GetWindowText(hTextBox, sBuf, iTextLen + 1);
		this->Append(sBuf, iTextLen);

		#ifdef _USE_GLOBAL_MEMPOOL
			pMem->Free(sBuf);
		#else
			free(sBuf);
		#endif

		return true;
	}

	return false;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Removes all white-spaces from the buffer.
*/
int CStringBuilder::RemoveWhiteSpaces(void)
{
	int iWPos = 0;

	for(int iRPos = 0; iRPos < this->Length; iRPos++)
	{
		if(!IsWhiteSpace(this->Buffer[iRPos]))
		{
			this->Buffer[iWPos++] = this->Buffer[iRPos];
		}
	}

	this->Buffer[iWPos] = '\0';

	this->Length = iWPos;

	return iWPos;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Removes white-spaces from the beginning and end of the buffer.
*/
int CStringBuilder::Trim(void)
{
	return(this->Length = NSWFL::String::CleanWhiteSpaces(this->Buffer, this->Length));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns the "Right Most" (x) number bytes form the buffer.
*/
int CStringBuilder::Right(char *sOutBuf, int iMaxSz, int iRight)
{
	return NSWFL::String::Right(sOutBuf, iMaxSz, this->Buffer, this->Length, iRight);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns the "Left Most" (x) nuber bytes form the buffer.
*/
int CStringBuilder::Left(char *sOutBuf, int iMaxSz, int iLeft)
{
	return NSWFL::String::Left(sOutBuf, iMaxSz, this->Buffer, this->Length, iLeft);
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Replaces a given piece of text with another. (not case-sensitive)
*/
int CStringBuilder::Replace(const char *sReplace, const char *sWith)
{
	return Replace(sReplace, sWith, false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Replaces a given piece of text with another allowing for optional case-sensitivity)
*/
int CStringBuilder::Replace(const char *sReplace, const char *sWith, bool bCaseSensitive)
{
	return Replace(sReplace, (int)strlen(sReplace), sWith, (int)strlen(sWith), bCaseSensitive);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Replaces a given piece of text with another allowing for optional case-sensitivity)
*/
int CStringBuilder::Replace(const char *sReplace, int iReplaceLen, const char *sWith, int iWithLen, bool bCaseSensitive)
{
	int iOccurrences = 0;
	int iPos = 0;
	int iMemDiff = (iWithLen - iReplaceLen);

	while(true)
	{
		if(bCaseSensitive)
		{
			iPos = this->InStr(sReplace, iPos);
		}
		else{
			iPos = InStrI(sReplace, iPos);
		}

		if(iPos >= 0)
		{
			if(iMemDiff > 0)
			{
				//Do we need to allocate additional RAM?
				if((this->Length + iMemDiff) >= this->Alloc)
				{
					if(!this->IncreaseSize(iMemDiff))
					{
						//Failed to allocate needed additional RAM.
						return false;
					}
				}

				int iCopyFrom = iPos + iMemDiff + iReplaceLen;
				for(int iWPos = (this->Length + iMemDiff) - 1; iWPos >= iCopyFrom; iWPos--)
				{
					this->Buffer[iWPos] = this->Buffer[iWPos - iMemDiff];
				}
			}
			else if(iMemDiff < 0){
				int iCopyFrom = (this->Length - iReplaceLen) + 1;
				for(int iWPos = iPos; iWPos < iCopyFrom; iWPos++)
				{
					this->Buffer[iWPos + iWithLen] = this->Buffer[iWPos + iReplaceLen];
				}
			}

			memcpy(this->Buffer + iPos, sWith, iWithLen);

			this->Length += iMemDiff;

			iPos++;
			iOccurrences++;
		}
		else{
			break;
		}
	}

	return iOccurrences;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Resizes the size of the data buffer.

	Note: Despite the name of the function, the additional bytes can be larger or smaller.
*/
bool CStringBuilder::IncreaseSize(int iAdditionalBytes)
{
	this->Alloc += (iAdditionalBytes + 1);
	
	#ifdef _USE_GLOBAL_MEMPOOL
		//Allocate more RAM.
		if((this->Buffer = (char *) pMem->ReAllocate(this->Buffer, sizeof(char), this->Alloc)) != NULL)
		{
			return true;
		}
	#else
		//Allocate more RAM.
		if((this->Buffer = (char *) realloc(this->Buffer, this->Alloc)) != NULL)
		{
			return true;
		}
	#endif

	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Sets the size of the data buffer.

	Note: Reducing the size of the buffer will truncate any data past the point of truncation.
*/
bool CStringBuilder::Resize(int iSize)
{
	if(iSize != this->Alloc)
	{
		this->Alloc = iSize;

		if(this->Alloc < this->Length)
		{
			this->Length = this->Alloc; //Truncate if necessary.
		}

		#ifdef _USE_GLOBAL_MEMPOOL
			//Allocate more RAM.
			if((this->Buffer = (char *) pMem->ReAllocate(this->Buffer, sizeof(char), this->Alloc)) != NULL)
			{
				return true;
			}
		#else
			//Allocate more RAM.
			if((this->Buffer = (char *) realloc(this->Buffer, this->Alloc)) != NULL)
			{
				return true;
			}
		#endif

		return false;
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Truncates the buffer, resetting it back to zero length.

	Note: This does not **necessarily** destroy all of the data.
*/
void CStringBuilder::Clear(void)
{
	if(!this->Buffer)
	{
		this->IncreaseSize(10);
	}

	this->Length = 0;
	this->Buffer[0] = '\0';
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	NULL terminates the data buffer at a specified location.
*/
bool CStringBuilder::Terminate(int iPosition)
{
	if(iPosition < this->Alloc)
	{
		this->Buffer[iPosition] = '\0';
		this->Length = iPosition;
		return true;
	}
	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	NULL terminates the data buffer.
*/
bool CStringBuilder::Terminate(void)
{
	bool bResult = this->Append("\0", 1);
	this->Length--; //We don't count the NULL terminator.
	return bResult;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends (char *) bytes of data of a specified length.
*/
bool CStringBuilder::Append(const char *sDat2Append, const int iDat2AppendSz)
{
	if(sDat2Append && iDat2AppendSz > 0)
	{
		int iRPos = 0;

		//Do we have enough RAM allocated for the newly read data?
		if((this->Length + iDat2AppendSz) + 1 > this->Alloc)
		{
			//Figure out how much more RAM we will need (And add a few more to it).
			if(!this->IncreaseSize(iDat2AppendSz + this->AllocAhead + 1))
			{
				return false;
			}
		}

		while(iRPos < iDat2AppendSz)
		{
			this->Buffer[this->Length++] = sDat2Append[iRPos++];
		}

		this->Buffer[this->Length] = '\0';
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends (unsigned char *) bytes of data of a specified length.
*/
bool CStringBuilder::Append(const unsigned char *sDat2Append, const int iDat2AppendSz)
{
	return this->Append((const char *)sDat2Append, iDat2AppendSz);
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends (unsigned char *) text. (must be NULL terminated)
*/
bool CStringBuilder::Append(const unsigned char *sDat2Append)
{
	if(sDat2Append)
	{
		return this->Append(sDat2Append, (int)strlen((const char *)sDat2Append));
	}
	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends (char *) text. (must be NULL terminated)
*/
bool CStringBuilder::Append(const char *sDat2Append)
{
	if(sDat2Append)
	{
		return this->Append(sDat2Append, (int)strlen(sDat2Append));
	}
	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends a single (char) byte.
*/
bool CStringBuilder::Append(const char cCharacter2Append)
{
	const char sText[1] = {cCharacter2Append};
	return this->Append(sText, 1);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends an unsigned C long to the data buffer.
*/
bool CStringBuilder::Append(const unsigned long iNum)
{
	char sNum[32];

	sprintf_s(sNum, sizeof(sNum), "%d", iNum);

	return this->Append(sNum);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends a C long to the data buffer.
*/
bool CStringBuilder::Append(const long iNum)
{
	char sNum[32];

	sprintf_s(sNum, sizeof(sNum), "%d", iNum);

	return Append(sNum);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends an unsigned C long to the data buffer.
*/
bool CStringBuilder::Append(const unsigned int iNum)
{
	char sNum[32];

	sprintf_s(sNum, sizeof(sNum), "%d", iNum);

	return this->Append(sNum);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends a C long to the data buffer.
*/
bool CStringBuilder::Append(const int iNum)
{
	char sNum[32];

	sprintf_s(sNum, sizeof(sNum), "%d", iNum);

	return this->Append(sNum);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends data to the data buffer from another instance of the class.
*/
bool CStringBuilder::Append(CStringBuilder *pBuffer)
{
	return this->Append(pBuffer->Buffer, pBuffer->Length);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends data to the data buffer from a file (by handle).
*/
bool CStringBuilder::AppendFromFileByHandle(FILE *hSource)
{
	int iBytesRead = 0;

	char sBuffer[CSTRINGBUILDER_GRANDULARITY + 1];

	do{
		iBytesRead = (int)fread(sBuffer, sizeof(char), CSTRINGBUILDER_GRANDULARITY, hSource);
		this->Append(sBuffer, iBytesRead);
	} while(iBytesRead == CSTRINGBUILDER_GRANDULARITY);

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends data to the data buffer from a file (by name).
*/
bool CStringBuilder::AppendFromFile(const char *sFileName)
{
	FILE *hSource = NULL;

	if(fopen_s(&hSource, sFileName, "rb") != 0)
	{
		return false;
	}

	bool bResult = this->AppendFromFileByHandle(hSource);

	fclose(hSource);

	return bResult;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends a 64 bit integer to the data buffer.
*/
bool CStringBuilder::Append(const __int64 i64Dat2Append)
{
	char sFloat[128];

	sprintf_s(sFloat, sizeof(sFloat), "%I64d", i64Dat2Append);

	return this->Append(sFloat, (int)strlen(sFloat));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends an unsigned 64 bit integer to the data buffer.
*/
bool CStringBuilder::Append(const unsigned __int64 i64Dat2Append)
{
	char sFloat[128];

	sprintf_s(sFloat, sizeof(sFloat), "%I64d", i64Dat2Append);

	return this->Append(sFloat, (int)strlen(sFloat));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends a floating point number to the data buffer (2 decimal places).
*/
bool CStringBuilder::Append(const float fDat2Append)
{
	return this->Append(fDat2Append, 2);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends a floating point number to the data buffer using with specified number of decimal places.
*/
bool CStringBuilder::Append(const float fDat2Append, int iDecimalPlaces)
{
	return this->Append((double)fDat2Append, iDecimalPlaces);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends a (double) floating point number to the data buffer (2 decimal places).
*/
bool CStringBuilder::Append(const double dDat2Append)
{
	return Append(dDat2Append, 2);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends a (double) floating point number to the data buffer using with specified number of decimal places.
*/
bool CStringBuilder::Append(const double dDat2Append, int iDecimalPlaces)
{
	char sFloat[64];
	char sFormat[10];

	sprintf_s(sFormat, sizeof(sFormat), "%%.%df", iDecimalPlaces);
	sprintf_s(sFloat, sizeof(sFloat), sFormat, dDat2Append);

	return this->Append(sFloat, (int)strlen(sFloat));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

char *CStringBuilder::ToString(void)
{
	if(this->Terminate())
	{
		return this->Buffer;
	}
	return NULL;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CStringBuilder::InStr(const char *sSearchFor, int sSearchForSz, const int iStartPos)
{
	return NSWFL::String::InStr(sSearchFor, sSearchForSz, this->Buffer, this->Length, iStartPos);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CStringBuilder::InStr(const char *sSearchFor, const int iStartPos)
{
	return NSWFL::String::InStr(sSearchFor, this->Buffer, this->Length, iStartPos);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CStringBuilder::InStr(const char *sSearchFor)
{
	return NSWFL::String::InStr(sSearchFor, this->Buffer, this->Length, 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CStringBuilder::InStrI(const char *sSearchFor, int sSearchForSz, const int iStartPos)
{
	return NSWFL::String::InStrI(sSearchFor, sSearchForSz, this->Buffer, this->Length, iStartPos);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CStringBuilder::InStrI(const char *sSearchFor, const int iStartPos)
{
	return NSWFL::String::InStrI(sSearchFor, this->Buffer, this->Length, iStartPos);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CStringBuilder::InStrI(const char *sSearchFor)
{
	return NSWFL::String::InStrI(sSearchFor, this->Buffer, this->Length, 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CStringBuilder::StrCmp(CStringBuilder *pStr)
{
	return strcmp(this->Buffer, pStr->Buffer);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CStringBuilder::StrCmpi(CStringBuilder *pStr)
{
	return _strcmpi(this->Buffer, pStr->Buffer);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CStringBuilder::StrnCmp(CStringBuilder *pStr, int iChars)
{
	return strncmp(this->Buffer, pStr->Buffer, iChars);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CStringBuilder::StrnCmpi(CStringBuilder *pStr, int iChars)
{
	return _strnicmp(this->Buffer, pStr->Buffer, iChars);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
